//
//  AddRecordingViewController.m
//  Blurbly
//
//  Created by Matthew Loseke on 10/3/13.
//  Copyright (c) 2013 Matthew Loseke. All rights reserved.
//

#import "AddRecordingViewController.h"

@interface AddRecordingViewController ()
{
    AVAudioRecorder *recorder;
    AVAudioSession *session;
    NSTimer *meterTimer;
    NSTimer *assignTimer;
    float dbLevel;
    float circScale;
    float lengthFloat;
    UIColor *topColor;
    UIColor *bottomColor;
    NSInteger assignInt;
    UIView *circleView;
    CGRect circleFrame;

    
}

@end

@implementation AddRecordingViewController

@synthesize recordButton, gradient, outputFileURL, recordedAudioFileName, nextButton;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //popup view
    
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"popDisplayed"] == FALSE)
    {
        [[NSUserDefaults standardUserDefaults] setBool:TRUE forKey:@"popDisplayed"];
        [[NSUserDefaults standardUserDefaults] synchronize];
    
    
    RNBlurModalView *modal = [[RNBlurModalView alloc] initWithViewController:self title:@"Info" message:@"Hold down the record button to start recording, you can release the button to stop recording and press it again to continue where you left off."];
    [modal show];
    }
    
    //creating circle view
    
    circleView = [[UIView alloc] initWithFrame:circleFrame];
    [self.view addSubview:circleView];
    circleView.backgroundColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
    
    
    circleFrame = CGRectMake(circleView.frame.origin.x, circleView.frame.origin.y, 220, 220);
    circleView.frame = circleFrame;
    circleView.layer.cornerRadius = 110;
    circleView.bounds = circleView.frame;
    
    circleView.center = CGPointMake(160,240);
    circleView.alpha = 0;
    circleView.autoresizesSubviews = YES;
    [self toScale:0];
    
    
    topColor = [UIColor clearColor];
    bottomColor = [UIColor clearColor];
    assignInt = 1;
    nextButton.enabled = NO;
    
    
    NSDate *date = [[NSDate alloc] initWithTimeIntervalSinceNow:1];
    recordedAudioFileName = [NSString stringWithFormat:@"%@", date];
    
    // sets the path for audio file (placeholder)
    NSArray *pathComponents = [NSArray arrayWithObjects:
                               [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject],
                               [NSString stringWithFormat:@"%@.m4a", recordedAudioFileName],
                               nil];
    outputFileURL = [NSURL fileURLWithPathComponents:pathComponents];
    
    
    
    // creates an audio session
    session = [AVAudioSession sharedInstance];
    [session setCategory:AVAudioSessionCategoryPlayAndRecord error:nil];
    
    // settings for the recorder
    NSMutableDictionary *recordSetting = [[NSMutableDictionary alloc] init];
    
    [recordSetting setValue:[NSNumber numberWithInt:kAudioFormatMPEG4AAC] forKey:AVFormatIDKey];
    [recordSetting setValue:[NSNumber numberWithFloat:44100.0] forKey:AVSampleRateKey];
    [recordSetting setValue:[NSNumber numberWithInt: 2] forKey:AVNumberOfChannelsKey];
    
    // initiate recorder
    NSError *error;
    recorder = [[AVAudioRecorder alloc] initWithURL:outputFileURL settings:recordSetting error:&error];
    recorder.delegate = self;
    [recorder prepareToRecord];
    recorder.meteringEnabled = YES;
    
    //gradient view stuff
    self.gradient = [[GradientView alloc] initWithFrame:circleView.bounds];
    self.gradient.autoresizingMask = UIViewAutoresizingFlexibleWidth |UIViewAutoresizingFlexibleHeight;
    self.gradient.layer.colors = @[ (id)topColor.CGColor, (id)bottomColor.CGColor];


}


#pragma mark making circles


-(void)buttonReleasedAnimation{
    
    
    [UIView animateWithDuration:0.25f animations:^(void) {
        circleView.transform = CGAffineTransformMakeScale (0, 0);
        circleView.backgroundColor = [UIColor lightGrayColor];
        
    }];
    
}



-(void)toScale:(float)newScale;
{
    
    [UIView animateWithDuration:0.25f animations:^(void) {
        circleView.transform = CGAffineTransformMakeScale (newScale, newScale);

    }];
}



- (void)animateLayer{

    [UIView animateWithDuration:0.25f delay:0.0f options:UIViewAnimationOptionCurveEaseInOut animations:^{

        gradient.layer.colors = @[(id)topColor.CGColor, (id)bottomColor.CGColor];
    }
                     completion:^(BOOL finished) {
                         }];
    
    
}

#pragma mark Audio Metering
- (void)startAudioMetering
{
    meterTimer = [NSTimer scheduledTimerWithTimeInterval:0.25
                                                       target:self
                                                     selector:@selector(updateAudioMeter)
                                                     userInfo:nil
                                                      repeats:YES];
}

- (void)stopAudioMetering
{

    [meterTimer invalidate];
}

- (void)updateAudioMeter //called by timer
{
    // audioRecorder being your instance of AVAudioRecorder
    [recorder updateMeters];
   dbLevel = [recorder averagePowerForChannel:0];
    NSLog(@"%f", dbLevel);
    
    
    //if statement determining the diameter of the circle
    if (dbLevel > -45.0 && dbLevel < -35.0) {
        circScale = 0.1f;
        lengthFloat = 30;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        gradient.alpha = 0.5f;
    }
    if (dbLevel > -35.0 && dbLevel < -30.0) {
        circScale = 0.25f;
        lengthFloat = 60;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        gradient.alpha = 0.6f;

    }
    if (dbLevel > -30.0 && dbLevel < -25.0) {
        circScale = 0.4f;
        lengthFloat = 80;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        gradient.alpha = 0.7f;

    }
    if (dbLevel > -25.0 && dbLevel < -20.0) {
        circScale = 0.55f;
        lengthFloat = 100;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        gradient.alpha = 0.8f;

    }
    if (dbLevel > -20.0 && dbLevel < -15.0) {
        circScale = 0.7f;
        lengthFloat = 120;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        gradient.alpha = 0.9f;

    }
    if (dbLevel > -15.0 && dbLevel < -10.0) {
        circScale = 8.5f;
        lengthFloat = 140;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        gradient.alpha = 1.0f;

    }
    if (dbLevel > -10.0 && dbLevel < 0) {
        circScale = 1.0f;
        lengthFloat = 160;
        topColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:75.0f/255.0f alpha:1.0f];
        bottomColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        gradient.alpha = 1.0f;

    }
    [self toScale:circScale];
    [self animateLayer];
}

#pragma mark view wave

-(void)growView: (UIView *)selectedView byLength:(float)newLength{
    
    CGRect tempRect = CGRectMake(selectedView.bounds.origin.x, selectedView.bounds.origin.y, 3, newLength);
    
    CGRect tempRect2 = CGRectMake(selectedView.frame.origin.x, selectedView.frame.origin.y, 3, 5);
    
    [UIView animateWithDuration:0.5 animations:^{
        selectedView.bounds = tempRect;
        
    } completion:^(BOOL finished) {
        [UIView animateWithDuration:1.9 animations:^(void) {
            
            
            selectedView.bounds = tempRect2;
            
        }];
    }];
    
    
    
}

-(void)assigningViewTimer{
    assignTimer = [NSTimer scheduledTimerWithTimeInterval:0.32857143
                                                   target:self
                                                 selector:@selector(assignView)
                                                 userInfo:nil
                                                  repeats:YES];
}

-(void)assignView{
    
    [self growView:[self.view viewWithTag:assignInt] byLength:20];
    assignInt++;
    
    if (assignInt > 70) {
        [assignTimer invalidate];
        assignInt = 1;
        
        RNBlurModalView *modal = [[RNBlurModalView alloc] initWithViewController:self title:@"Alert" message:@"Maximum recording time reached, press Next to continue or press the recording button to try again."];
        [modal show];
    }

}

#pragma mark button stuff

- (IBAction)gotoFeedView:(id)sender {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)buttonPressedStartRecording:(id)startRecording {
    [recorder recordForDuration:23];
    [self startAudioMetering];
    [self assigningViewTimer];
    circleView.alpha = 1.0;
    nextButton.enabled = YES;
    circleView.backgroundColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
}

- (IBAction)buttonReleasedStopRecording:(id)stopRecordingDefault {
    [recorder pause];
    [self stopAudioMetering];
    [assignTimer invalidate];
    circleView.alpha = 0;
    [self toScale:0];
}

- (IBAction)buttonReleasedOutsideStopRecording:(id)stopRecodingBackup {
    [recorder pause];
    [self stopAudioMetering];
    [assignTimer invalidate];
    circleView.alpha = 0;
    [self toScale:0];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    circleView.hidden =YES;
    [recorder stop];
    FinalizeRecordingViewController *vc = segue.destinationViewController;
    vc.recordedAudioURL = outputFileURL;
    // Parse gives each file upload a unique identifier, this name is required, but arbitrary.
    vc.recordedAudioFileName = @"foo";
    [[UINavigationBar appearance] setBackIndicatorImage:[UIImage imageNamed:@"button-play.jpg"]];
}
@end
