//
//  TableViewController.m
//  Blurbly
//
//  Created by Matthew Loseke on 10/22/13.
//  Copyright (c) 2013 Matthew Loseke. All rights reserved.
//


#import "TableViewController.h"


@interface TableViewController ()
@end

@implementation TableViewController

@synthesize player, timerForAudioClip, selectedCell, playOrPauseButton, audioClips;

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.navigationController setToolbarHidden:NO];
    
    // Add the 'menu' button to nav
    UIImage *burger = [UIImage imageNamed:@"burger"];
    UIBarButtonItem *button = [[UIBarButtonItem alloc] initWithImage:burger landscapeImagePhone:burger style:UIBarButtonItemStylePlain target:self.revealViewController action:@selector(revealToggle:)];
    button.tintColor = [UIColor colorWithRed:172.0f/255.0f green:174.0f/255.0f blue:177.0f/255.0f alpha:1.0f];
    
    [self.navigationItem setLeftBarButtonItem:button];
    // Set the gesture - this is so we can swipe from the left edge of screen, to the right, and have reveal happen
    [self.view addGestureRecognizer:self.revealViewController.panGestureRecognizer];
    
    [self loadDataFromParse];
    
    //setting up refresh
    UIRefreshControl *refresh = [[UIRefreshControl alloc] init];
    refresh.attributedTitle = [[NSAttributedString alloc] initWithString:@"Pull to Refresh"];
    [refresh addTarget:self
                action:@selector(refreshView:)
      forControlEvents:UIControlEventValueChanged];
    self.refreshControl = refresh;
    
    self.clearsSelectionOnViewWillAppear = NO;
}

#pragma mark - API

- (void)loadDataFromParse
{
    PFQuery *query = [PFQuery queryWithClassName:@"AudioClip"];
    [query findObjectsInBackgroundWithBlock:^(NSArray *objects, NSError *error) {
        if (!error) {
            NSArray *reverse = [[objects reverseObjectEnumerator] allObjects];

            audioClips = [[NSArray alloc] initWithArray:reverse];
            [self.tableView reloadData];
            for (PFObject *object in audioClips) {
                PFFile *audioFile = [object objectForKey:@"audio"];
                [audioFile getDataInBackgroundWithBlock:^(NSData *data, NSError *error) {
                    [object setObject:data forKey:@"audio"];
                }];
                [self.tableView reloadData];
            }
        } else {
            // Log details of the failure
            NSLog(@"Error: %@ %@", error, [error userInfo]);
        }
    }];
}

#pragma mark - Refreshing TableView

-(void)refreshView:(UIRefreshControl *)refresh {
    refresh.attributedTitle = [[NSAttributedString alloc] initWithString:@"Refreshing data..."];
    
    [self loadDataFromParse];
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"MMM d, h:mm a"];
    NSString *lastUpdated = [NSString stringWithFormat:@"Last updated on %@",
                             [formatter stringFromDate:[NSDate date]]];
    refresh.attributedTitle = [[NSAttributedString alloc] initWithString:lastUpdated];
    [refresh endRefreshing];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return audioClips.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *identifier =@"blurb";
    
    UITableViewCell *cell = (UITableViewCell *)[tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath];
    
    cell.selectedBackgroundView = [UIView new];
    
    // this color should be pulled out into a setting or something
    cell.selectedBackgroundView.backgroundColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
    
    PFObject *audioClip = audioClips[indexPath.row];
    
    UIImageView *imgView = (UIImageView *)[cell viewWithTag:4];
    [imgView setBackgroundColor:[UIColor colorWithRed:242.0f/255.0f green:242.0f/255.0f blue:242.0f/255.0f alpha:1]];
    
    int randomInteger = arc4random() % 10;
    NSString *imageString = [NSString stringWithFormat:@"avatar_%i", randomInteger];
    
    imgView.image = [UIImage imageNamed:imageString];
    [imgView.layer setBorderColor:[[UIColor whiteColor] CGColor]];
    [imgView.layer setBorderWidth:2.0];
    
    PFUser *user = [audioClip objectForKey:@"user"];
    [user fetchIfNeededInBackgroundWithBlock:^(PFObject *object, NSError *error) {
        UILabel *label;
        label = (UILabel *)[cell viewWithTag:1];
        label.text = [NSString stringWithFormat:@"%@", [user objectForKey:@"username"]];
    }];
    
    UILabel *label;
    
    label = (UILabel *)[cell viewWithTag:2];
    label.text = [NSString stringWithFormat:@"%@", [audioClip.createdAt timeAgo]];

    label = (UILabel *)[cell viewWithTag:3];
    label.text = [NSString stringWithFormat:@"%@", [audioClip objectForKey:@"title"]];
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    selectedCell = [tableView cellForRowAtIndexPath:indexPath];
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [[selectedCell viewWithTag:5] removeFromSuperview];
}

#pragma mark - Audio

#pragma mark view

- (void)configureAnimationForAudioPlay
{
    timerForAudioClip = [NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(widenItJustALittleBit:) userInfo:nil repeats:YES];
    UIView *playbackBar = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, 3)];
    [playbackBar setTag:5];
    [self.navigationController.toolbar addSubview: playbackBar];
    timerForAudioClip = [NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(widenItJustALittleBit:) userInfo:nil repeats:YES];
}

- (void)widenItJustALittleBit: (NSTimer *)timer
{
    float toolbarWidth = self.navigationController.toolbar.bounds.size.width;

    if (player) {
        float lengthOfAudio = player.duration;
        float currentTime = player.currentTime;
        float percentagePlayed = currentTime / lengthOfAudio;
        float playbackBarWidth = percentagePlayed * toolbarWidth;
        UIView *playbackBar = [self.navigationController.toolbar viewWithTag:5];
        playbackBar.backgroundColor = [UIColor colorWithRed:252.0f/255.0f green:63.0f/255.0f blue:67.0f/255.0f alpha:1.0f];
        [playbackBar setFrame:CGRectMake(0, 0, playbackBarWidth, 3)];
    }
}

- (void)removePlaybackBar
{
    UIView *playbackBar = [self.navigationController.toolbar viewWithTag:5];
    [playbackBar removeFromSuperview];
}

#pragma mark control

- (IBAction)playButton:(id)sender {
   
    if (player.playing) {
        [player pause];
    } else {
        [player play];
    }
   
    // The controller's instance of player is set to nil everytime an audio clip finishes playing.
    // When a new cell is selected we re-alloc/init with its audioClip's audio.
    if (!player) {
        [self configurePlayerWithSelectedCellsAudio];
        [player play];
        [self configureAnimationForAudioPlay];
    }
}

- (void)configurePlayerWithSelectedCellsAudio
{
    NSIndexPath *indexPath = [self.tableView indexPathForSelectedRow];
    if (indexPath) {
        PFObject *audioClip = audioClips[indexPath.row];
        NSError *error;
        player = [[AVAudioPlayer alloc] initWithData:[audioClip objectForKey:@"audio"] error:&error];
        player.delegate = self;
    }
}

#pragma mark AVAudioPlayerDelegate method

- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag
{
    [self removePlaybackBar];
    [timerForAudioClip invalidate];
    self.player = nil;
}

#pragma mark - NSFetchedResultsControllerDelegate

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type newIndexPath:(NSIndexPath *)newIndexPath
{
    [self.tableView reloadData];
}

@end
